********************************************************************************
*                               NyanCat pour MO6                               *
********************************************************************************
* Auteur  : OlivierP (sauf si indication contraire)                            *
* Date    : novembre-dcembre 2011 / fvrier 2016                              *
* Licence : GNU GPLv3 (http://www.gnu.org/copyleft/gpl.html)                   *
********************************************************************************

(main)NyanCat.asm

   ORG $4000

ADR_ARC EQU $0BED
ADR_MIX EQU ADR_ARC+4
ADR_CAT EQU ADR_MIX+3

   ORCC #80        * ne pas interrompre

********************************************************************************
* Definition de la palette de couleurs
* 
* Provenance de ce bout de code : manuel technique des TO8, TO9, TO9+ page 70
* modifi par OlivierP pour faire les 16 valeurs
********************************************************************************
   CLRA
PALETTE_FOND
   PSHS A
   ASLA
   STA $A7DB
   LDD #$0310      * on force toutes les couleurs durant l'initialisation
   STB $A7DA
   STA $A7DA
   PULS A
   INCA
   CMPA #$F
   BNE PALETTE_FOND


********************************************************************************
* Passage en mode 160x200x16c
* 
* Provenance de ce bout de code :  manuel technique des TO8, TO9, TO9+ page 61
********************************************************************************
   LDA #$7B
   STA $A7DC


********************************************************************************
* Initialisation de la routine de commutation de page video
* 
* Provenance de ce bout de code :  http://pulsdemos.com/vector02.html
* Adaptation MO6 par OlivierP. 
********************************************************************************
   LDB $2081
   ORB #$50     * RAM dans espace cartouche par A7E6 (bit 6 de A7E7  1), commutation banque par A7E5 (bit 4 de A7E7  1)
   STB $2081
   STB $A7E7


********************************************************************************
* Definition de la palette de couleurs
* 
* Provenance de ce bout de code : manuel technique des TO8, TO9, TO9+ page 70
* amlior par OlivierP pour grer un tableau de valeurs
********************************************************************************
   LDY #PALET
   CLRA
PALETTE
   PSHS A
   ASLA
   STA $A7DB
   LDD ,Y++
   STB $A7DA
   STA $A7DA
   PULS A
   INCA
   CMPY #FINPA
   BNE PALETTE


********************************************************************************
* Effacement ecran (les deux pages)
********************************************************************************
   JSR SCRC
   JSR EFF
   JSR SCRC
   JSR EFF


********************************************************************************
* Boucle principale
********************************************************************************
BOUCLE_PRINC

   LDX #ANIM1
   LDY #ANIM2
   JSR FRAME_ANIM_MO6
   JSR SCRC        * changement de page cran

   LDX #ANIM1
   LDY #ANIM2
   JSR FRAME_ANIM
   JSR SCRC        * changement de page cran

   BRA BOUCLE_PRINC


********************************************************************************
* Changement de page cran
* 
* Provenance de ce bout de code :  http://pulsdemos.com/vector02.html
* Adaptation MO6 par OlivierP.
********************************************************************************
SCRC
   LDB SCRC0+1
   ANDB #$80          * BANK1 utilise ou pas pour l'affichage
   ORB #$0A           * contour cran = couleur A
   STB $A7DD
   COM SCRC0+1
SCRC0
   LDB #$00
   ANDB #$02          * page RAM no0 ou no2 utilise dans l'espace cartouche
   ORB #$60           * criture autorise / espace cartouche recouvert par RAM
   STB $A7E6
   RTS


********************************************************************************
* Effacement de l'cran
********************************************************************************
EFF
   LDA #$AA  * couleur fond
   LDY #$B000
EFF_RAM
   STA ,Y+
   CMPY #$EFFF
   BNE EFF_RAM
   RTS


********************************************************************************
* Affichage d'une frame
* 
* La RAM video est compose de la faon suivante
* en octet en RAMA code 2 pixels (4 bits par pixel)
* en octet en RAMB code les 2 pixels suivants (4 bits par pixel)
********************************************************************************
FRAME_ANIM
   TFR Y,D         * D (A et B) contient maintenant l'adresse de fin du tableau ANIM (Y)
   STA FRAME_ANIM_YVAL+1    * modifie le CMPX de FRAME_ANIM_YVAL
   STB FRAME_ANIM_YVAL+2    * modifie le CMPX de FRAME_ANIM_YVAL

FRAME_ANIM_B
   PSHS X          * X contient le pointeur dans le tableau ANIM
   LDD ,X++        * D contient les donnees graphiques  
   LDY ,X++        * Y contient l'adresse video
   TFR D,X         * X contient les donnees graphiques

* Affichage d'un bitmap en mode 16 couleurs
* On affiche chaque point en double (sur la position courante en RAM video
* ainsi que sur la ligne suivante) afin d'avoir des pixels carrs.
* 0xEE indique la fin de ligne (on perd donc une couleur possible).
* Il y a ensuite une valeur qui permet de pointer vers le dbut de la ligne
* suivante en RAM video. Si cette valeur en nulle alors le dessin est fini.
* Cette routine ne gre pas la transparence.
* X contient le bitmap, Y pointe vers la ram video

   LDD ,X++        * D (A et B) contient le point bitmap
DESSIN_B
   STA $B000,Y     * ecrit les 2 points contenus dans A en RAMA
   STA $B028,Y     * ecrit les 2 points contenus dans A sur la ligne suivante en RAMA
   STB $D000,Y     * ecrit les 2 points contenus dans B en RAMB
   STB $D028,Y     * ecrit les 2 points contenus dans B sur la ligne suivante en RAMB
   LDD ,X++        * D (A et B) contient le point bitmap suivant
   LEAY 1,Y
   CMPA #$EE       * EE = fin de ligne
   BNE DESSIN_B    * aller  DESSIN_B si pas fin de ligne
   CMPB #$00
   BEQ DESSIN_FIN  * sortir si fin bitmap
   LEAY B,Y        * ajout B pour que Y pointe vers la ligne suivante
   LDD ,X++        * doublon de code plutot que BRA DESSIN
   BRA DESSIN_B
DESSIN_FIN
   PULS X          * X contient le pointeur dans le tableau ANIM
   LEAX 4,X        * X pointe vers l'anim suivante dans le tableau ANIM
FRAME_ANIM_YVAL
   CMPX #$0000     * cette valeur est initialise entre FRAME_ANIM et FRAME_ANIM_B  
   BNE FRAME_ANIM_B  * boucler si c'est n'est pas la derniere anim

   RTS


********************************************************************************
* Affichage d'une frame sur MO6
* 
* La RAM video est compose de la faon suivante
* en octet en RAMA code 2 pixels (4 bits par pixel)
* en octet en RAMB code les 2 pixels suivants (4 bits par pixel)
********************************************************************************
FRAME_ANIM_MO6
   TFR Y,D         * D (A et B) contient maintenant l'adresse de fin du tableau ANIM (Y)
   STA FRAME_ANIM_MO6_YVAL+1    * modifie le CMPX de FRAME_ANIM_YVAL
   STB FRAME_ANIM_MO6_YVAL+2    * modifie le CMPX de FRAME_ANIM_YVAL

FRAME_ANIM_MO6_B
   PSHS X          * X contient le pointeur dans le tableau ANIM
   LDD ,X++        * D contient les donnees graphiques  
   LDY ,X++        * Y contient l'adresse video
   TFR D,X         * X contient les donnees graphiques

* Affichage d'un bitmap en mode 16 couleurs
* On affiche chaque point en double (sur la position courante en RAM video
* ainsi que sur la ligne suivante) afin d'avoir des pixels carrs.
* 0xEE indique la fin de ligne (on perd donc une couleur possible).
* Il y a ensuite une valeur qui permet de pointer vers le dbut de la ligne
* suivante en RAM video. Si cette valeur en nulle alors le dessin est fini.
* Cette routine ne gre pas la transparence.
* X contient le bitmap, Y pointe vers la ram video

   LDD ,X++        * D (A et B) contient le point bitmap
DESSIN_MO6
   CMPY #$1000
   BGT DESSIN_MO6_HACK
   STA $C000,Y     * ecrit les 2 points contenus dans A en RAMA
   STA $C028,Y     * ecrit les 2 points contenus dans A sur la ligne suivante en RAMA
   STB $E000,Y     * ecrit les 2 points contenus dans B en RAMB
   STB $E028,Y     * ecrit les 2 points contenus dans B sur la ligne suivante en RAMB
   BRA DESSIN_MO6_HACK_FIN
DESSIN_MO6_HACK
   * Y est > $1000, on crit donc en $B000 et $D000
   STA $C000,Y     * ecrit les 2 points contenus dans A en RAMA
   STA $C028,Y     * ecrit les 2 points contenus dans A sur la ligne suivante en RAMA
   STB $A000,Y     * ecrit les 2 points contenus dans B en RAMB
   STB $A028,Y     * ecrit les 2 points contenus dans B sur la ligne suivante en RAMB
DESSIN_MO6_HACK_FIN
   LDD ,X++        * D (A et B) contient le point bitmap suivant
   LEAY 1,Y
   CMPA #$EE       * EE = fin de ligne
   BNE DESSIN_MO6  * aller  DESSIN_MO6 si pas fin de ligne
   CMPB #$00
   BEQ DESSIN_MO6_FIN  * sortir si fin bitmap
   LEAY B,Y        * ajout B pour que Y pointe vers la ligne suivante
   LDD ,X++        * doublon de code plutot que BRA DESSIN
   BRA DESSIN_MO6
DESSIN_MO6_FIN
   PULS X          * X contient le pointeur dans le tableau ANIM
   LEAX 4,X        * X pointe vers l'anim suivante dans le tableau ANIM
FRAME_ANIM_MO6_YVAL
   CMPX #$0000     * cette valeur est initialise entre FRAME_ANIM et FRAME_ANIM_B  
   BNE FRAME_ANIM_MO6_B  * boucler si c'est n'est pas la derniere anim

   RTS


********************************************************************************
* DONNEES                                                 B  V  R
********************************************************************************
PALET
   FDB $0FFF    * blanc                                   FF FF FF
   FDB $00EE    * jaune                                   00 FF FF
   FDB $05AF    * saumon (contour du chat)                99 CC FF
   FDB $00E1    * vert clair                              00 FF 33
   FDB $0F5F    * rose clair (milieu du chat)             FF 99 FF
   FDB $044F    * saumon rose (joues du chat)             99 99 FF
   FDB $005F    * orange                                  00 99 FF
   FDB $0555    * gris                                    99 99 99
   FDB $0F50    * bleu                                    FF 99 00
   FDB $051F    * rose fonc (points du chat)             99 33 FF
   FDB $0310    * bleu fonc (fond cran)                 FF 33 66
   FDB $000F    * rouge                                   00 00 FF
   FDB $0000    * noir                                    00 00 00
   FDB $0F13    * violet                                  66 33 00
   FDB $0000    * non utilis / fin de ligne
   FDB $0310    * bleu fonc (fond cran pour dessin_tr)
FINPA

ANIM1
   FDB ARC1
   FDB ADR_ARC
   FDB MIX1
   FDB ADR_MIX
   FDB CAT1
   FDB ADR_CAT

ANIM2

ARC1
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4c

   FCB $aa
   FCB $ab
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $ba
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4c

   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $EE
   FCB $4c

   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $EE
   FCB $4c

   FCB $bb
   FCB $b6
   FCB $66
   FCB $66
   FCB $66
   FCB $6b
   FCB $bb
   FCB $bb
   FCB $EE
   FCB $4c

   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $EE
   FCB $4c

   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $EE
   FCB $4c

   FCB $66
   FCB $61
   FCB $11
   FCB $11
   FCB $11
   FCB $16
   FCB $66
   FCB $66
   FCB $EE
   FCB $4c

   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $EE
   FCB $4c

   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $11
   FCB $EE
   FCB $4c

   FCB $11
   FCB $13
   FCB $33
   FCB $33
   FCB $33
   FCB $31
   FCB $11
   FCB $11
   FCB $EE
   FCB $4c

   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $EE
   FCB $4c

   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $33
   FCB $EE
   FCB $4c

   FCB $33
   FCB $38
   FCB $88
   FCB $88
   FCB $88
   FCB $83
   FCB $33
   FCB $33
   FCB $EE
   FCB $4c

   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $EE
   FCB $4c

   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $EE
   FCB $4c

   FCB $88
   FCB $8d
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $d8
   FCB $88
   FCB $88
   FCB $EE
   FCB $4c

   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $EE
   FCB $4c

   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dd
   FCB $EE
   FCB $4c

   FCB $dd
   FCB $da
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $ad
   FCB $dd
   FCB $dd
   FCB $EE
   FCB $4c

   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $00

MIX1
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $ac
   FCB $EE
   FCB $4d

   FCB $aa
   FCB $ab
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $c2
   FCB $EE
   FCB $4d

   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bc
   FCB $22
   FCB $EE
   FCB $4d

   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bb
   FCB $bc
   FCB $22
   FCB $EE
   FCB $4d

   FCB $bb
   FCB $b6
   FCB $66
   FCB $66
   FCB $6c
   FCB $24
   FCB $EE
   FCB $4d

   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $6c
   FCB $24
   FCB $EE
   FCB $4d

   FCB $66
   FCB $66
   FCB $66
   FCB $66
   FCB $6c
   FCB $24
   FCB $EE
   FCB $4d

   FCB $66
   FCB $6c
   FCB $cc
   FCB $c1
   FCB $1c
   FCB $24
   FCB $EE
   FCB $4d

   FCB $11
   FCB $1c
   FCB $77
   FCB $cc
   FCB $1c
   FCB $24
   FCB $EE
   FCB $4d

   FCB $11
   FCB $1c
   FCB $c7
   FCB $7c
   FCB $cc
   FCB $24
   FCB $EE
   FCB $4d

   FCB $11
   FCB $13
   FCB $cc
   FCB $77
   FCB $cc
   FCB $24
   FCB $EE
   FCB $4d

   FCB $33
   FCB $33
   FCB $3c
   FCB $c7
   FCB $7c
   FCB $24
   FCB $EE
   FCB $4d

   FCB $33
   FCB $33
   FCB $33
   FCB $cc
   FCB $cc
   FCB $24
   FCB $EE
   FCB $4d

   FCB $33
   FCB $38
   FCB $88
   FCB $88
   FCB $cc
   FCB $24
   FCB $EE
   FCB $4d

   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $8c
   FCB $22
   FCB $EE
   FCB $4d

   FCB $88
   FCB $88
   FCB $88
   FCB $88
   FCB $8c
   FCB $22
   FCB $EE
   FCB $4d

   FCB $88
   FCB $8d
   FCB $dd
   FCB $dd
   FCB $cc
   FCB $c2
   FCB $EE
   FCB $4d

   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dc
   FCB $77
   FCB $7c
   FCB $EE
   FCB $4d

   FCB $dd
   FCB $dd
   FCB $dd
   FCB $dc
   FCB $77
   FCB $cc
   FCB $EE
   FCB $4d

   FCB $dd
   FCB $da
   FCB $aa
   FCB $ac
   FCB $cc
   FCB $ca
   FCB $EE
   FCB $4d

   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $00

CAT1
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $22
   FCB $22
   FCB $22
   FCB $22
   FCB $22
   FCB $22
   FCB $22
   FCB $22
   FCB $ca
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $24
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $22
   FCB $2c
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $44
   FCB $94
   FCB $49
   FCB $44
   FCB $44
   FCB $42
   FCB $2c
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $49
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $2c
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $4c
   FCB $c4
   FCB $49
   FCB $44
   FCB $2c
   FCB $ac
   FCB $ca
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $c7
   FCB $7c
   FCB $44
   FCB $44
   FCB $2c
   FCB $c7
   FCB $7c
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $49
   FCB $44
   FCB $c7
   FCB $77
   FCB $c4
   FCB $44
   FCB $2c
   FCB $77
   FCB $7c
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $44
   FCB $44
   FCB $c7
   FCB $77
   FCB $7c
   FCB $cc
   FCB $c7
   FCB $77
   FCB $7c
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $94
   FCB $44
   FCB $44
   FCB $c7
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $7c
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $44
   FCB $9c
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $ca
   FCB $EE
   FCB $4a

   FCB $94
   FCB $44
   FCB $44
   FCB $4c
   FCB $77
   FCB $70
   FCB $c7
   FCB $77
   FCB $77
   FCB $0c
   FCB $77
   FCB $ca
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $44
   FCB $4c
   FCB $77
   FCB $7c
   FCB $c7
   FCB $77
   FCB $c7
   FCB $cc
   FCB $77
   FCB $ca
   FCB $EE
   FCB $4a

   FCB $44
   FCB $44
   FCB $94
   FCB $4c
   FCB $75
   FCB $57
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $55
   FCB $ca
   FCB $EE
   FCB $4a

   FCB $49
   FCB $44
   FCB $44
   FCB $4c
   FCB $75
   FCB $57
   FCB $c7
   FCB $7c
   FCB $77
   FCB $c7
   FCB $55
   FCB $ca
   FCB $EE
   FCB $4a

   FCB $24
   FCB $44
   FCB $44
   FCB $44
   FCB $c7
   FCB $77
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $c7
   FCB $7c
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $22
   FCB $22
   FCB $22
   FCB $22
   FCB $2c
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $77
   FCB $ca
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $cc
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $ac
   FCB $77
   FCB $ca
   FCB $aa
   FCB $aa
   FCB $c7
   FCB $7c
   FCB $ac
   FCB $77
   FCB $ca
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $ac
   FCB $cc
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $ac
   FCB $cc
   FCB $aa
   FCB $cc
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $4a

   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $aa
   FCB $EE
   FCB $00

